
clear
format long 

%=================================================================
%   This program computesx all six heteroclinic intersections 
%   by a classical continuation method.  This code produces the 
%   six intersections used in the paper.
%  
%================================================================


%==============================================================
%Computation Controls:
%==============================================================


%Map Parameters:
a = 0.44;
b = 0.21;
c = 0.35;

alpha = -0.25; 
tau = -0.3;   

  
%Order of manifold computation:
N = 50;

%Number of times to globalize best disks:
k = 9
 
%number of times to globalize heteroclinic connections:
K_hetero = 35

%Domain parameters:
  R = 0.7
  allowed_error = 5*10^(-15)      %10^-14 definitely works.
  num_theta = 100
  num_r = 40
  
%path parameters
  pathStep = 1.0*10^(-3);      %2.5*10^(-3)
  epsilon_path = 10^(-14);
  maxTry = 10000;
  
%Set Parameters:
  intersectionTol = 2.5*10^(-2) 

  
%==============================================================
%Compute Best Domain:
%==============================================================


[Au, Bu, Cu, As, Bs, Cs, Du, Ds, Wu_loc, Ws_loc, x0, x1, p0, p1, lambda_s, lambda_u] = ...
    bestDomainFunction(a, b, c, alpha, ...
              tau, N, R, allowed_error, num_theta, num_r);

%Plane Data
  
%Point q is a point on the plane:
q0 = p0 + 0.5*(p1-p0);

%eta one and two are the direction vectors of the plane:        
eta1 = [1; 0; -1]
eta2 = [0; 1; -1]
  
%Iterate Disks:        

Wu = iterate_set_k(Wu_loc, k, alpha, tau, a, b, c);
Ws = inverse_iterate_set_k(Ws_loc, k, alpha, tau, a, b, c);

%=========================================================================
%Set Operations:
%=========================================================================

intersectPlane_u = setIntersectPlane(Wu, eta1, eta2, q0, intersectionTol);
intersectPlane_s = setIntersectPlane(Ws, eta1, eta2, q0, intersectionTol);

[intersectionGuesses_u, intersectionGuesses_s] = ... 
    stableUnstableIntersection(intersectPlane_u, intersectPlane_s, 2*intersectionTol)

%=======================================================================
%Find Guesses:
%=======================================================================

[guesses_uParameters, guesses_uIterates, k_u, trashI] = findUnstableParameters(intersectionGuesses_u, ...
    Du, Wu_loc, lambda_u, alpha, tau, a, b, c);


[guesses_sParameters, guesses_sIterates, k_s, trashII] = findStableParameters(intersectionGuesses_s, ...
    Ds, Ws_loc, lambda_s, alpha, tau, a, b, c);



%====================================================================
%Tighten Up Guesses:
%====================================================================

temp = size(guesses_uParameters);
numGuesses = temp(1)
epsilon_ic = 10^(-14)
maxTry = 1000;


for k = 1:numGuesses
[new_guesses_uParameters(k, 1:2), new_guesses_sParameters(k, 1:2)] = ...
    initialGuessNewton(N, k_u, k_s, epsilon_ic, ...
    guesses_uParameters(k,1:2), guesses_sParameters(k, 1:2), Au, Bu, Cu, As, Bs, Cs, ...
    alpha, tau, a, b, c, maxTry);
end


for j = 1:6
  Pu = evaluate_complexCase(new_guesses_uParameters(j,1), new_guesses_uParameters(j, 2), Au, Bu, Cu, N)';
  Ps = evaluate_complexCase(new_guesses_sParameters(j,1), new_guesses_sParameters(j, 2), As, Bs, Cs, N)';
  F1 = iterate_point_k(Pu, k_u, alpha, tau, a, b, c); 
  F2 = inverse_iterate_point_k(Ps, k_s, alpha, tau, a, b, c);      
  intersectionCheck_u(j,1:3) = F1';
  intersectionCheck_s(j,1:3) = F2';
end

figure
hold on

plot3(intersectPlane_u(:,1), intersectPlane_u(:,2), intersectPlane_u(:, 3), 'b.');
plot3(intersectPlane_s(:,1), intersectPlane_s(:,2), intersectPlane_s(:, 3), 'r.');
plot3(x0, x0, x0, 'g*')
plot3(x1, x1, x1, 'g*')
plot3(intersectionGuesses_u(:,1), intersectionGuesses_u(:,2), intersectionGuesses_u(:,3), 'ko');
plot3(intersectionGuesses_s(:,1), intersectionGuesses_s(:,2), intersectionGuesses_s(:,3), 'mo');

figure
hold on

plot3(intersectPlane_u(:,1), intersectPlane_u(:,2), intersectPlane_u(:, 3), 'b.');
plot3(intersectPlane_s(:,1), intersectPlane_s(:,2), intersectPlane_s(:, 3), 'r.');
plot3(x0, x0, x0, 'g*')
plot3(x1, x1, x1, 'g*')
plot3(intersectionCheck_u(:,1), intersectionCheck_u(:,2), intersectionCheck_u(:,3), 'ko');


figure
hold on

plot3(intersectPlane_u(:,1), intersectPlane_u(:,2), intersectPlane_u(:, 3), 'b.');
plot3(intersectPlane_s(:,1), intersectPlane_s(:,2), intersectPlane_s(:, 3), 'r.');
plot3(x0, x0, x0, 'g*')
plot3(x1, x1, x1, 'g*')
plot3(intersectionCheck_s(:,1), intersectionCheck_s(:,2), intersectionCheck_s(:,3), 'mo');

k_u
k_s

%X = U



%===================================================================
%Make Guess Field:
%===================================================================

u_guessField = new_guesses_uParameters;
s_guessField = new_guesses_sParameters;

i = sqrt(-1);

for k = 1:numGuesses
    s_u = new_guesses_uParameters(k, 1);
    t_u = new_guesses_uParameters(k, 2);
    s_s = new_guesses_sParameters(k,1);
    t_s = new_guesses_sParameters(k, 2);
    z_u0 = s_u + i*t_u;
    z_s0 = s_s + i*t_s;
    for j = 1:15
        z_u = lambda_u^(j)*z_u0;
        z_uInv = lambda_u^(-j)*z_u0;
        z_s = lambda_s^(j)*z_s0;
        z_sInv = lambda_s^(-j)*z_s0;
        this_u = [real(z_u), imag(z_u)];
        this_u_inv = [real(z_uInv), imag(z_uInv)];
        this_s = [real(z_s), imag(z_s)];
        this_s_inv = [real(z_sInv), imag(z_sInv)];
        u_guessField = [u_guessField; this_u; this_u_inv];
        s_guessField = [s_guessField; this_s; this_s_inv];
    end
end    


%===================================================================
%Evaluate Guesses:
%===================================================================

for i = 1:numGuesses
   z_n(1) = new_guesses_uParameters(i, 1);
   z_n(2) = new_guesses_uParameters(i, 2);
   z_n(3) = new_guesses_sParameters(i, 1);
   z_n(4) = new_guesses_sParameters(i, 2); 
   
   Pu = evaluate_complexCase(z_n(1), z_n(2), Au, Bu, Cu, N)';
   Ps = evaluate_complexCase(z_n(3), z_n(4), As, Bs, Cs, N)';
   F1 = iterate_point_k(Pu, k_u, alpha, tau, a, b, c); 
   F2 = inverse_iterate_point_k(Ps, k_s, alpha, tau, a, b, c);
      
   F(i) = norm(F1 - F2);
end

F'

%================================================================
%Continuation:
%================================================================


[Path, uParSpacePath, sParSpacePath] = ...
      continue_heteroclinic_iterate(N, k_u, k_s, u_guessField, s_guessField, ... 
      pathStep, epsilon_path, new_guesses_uParameters(1, 1:2), new_guesses_sParameters(1, 1:2), ...
      Au, Bu, Cu, As, Bs, Cs, alpha, tau, a, b, c, maxTry);

for k = 2:numGuesses
  [temp_Path, temp_uParSpacePath, temp_sParSpacePath] = ...
      continue_heteroclinic_iterate(N, k_u, k_s, u_guessField, s_guessField, ... 
      pathStep, epsilon_path, new_guesses_uParameters(k, 1:2), new_guesses_sParameters(k, 1:2), ...
      Au, Bu, Cu, As, Bs, Cs, alpha, tau, a, b, c, maxTry);
 
  Path = [Path; temp_Path];
  uParSpacePath = [uParSpacePath; temp_uParSpacePath];
  sParSpacePath = [sParSpacePath; temp_sParSpacePath];
end

continuedPath = iterate_set_k(Path, K_hetero, alpha, tau, a, b, c);
continuedPathInv = inverse_iterate_set_k(Path, K_hetero, alpha, tau, a, b, c);

temp = size(Path);
pathPoints = temp(1);
badIndicesFound = 0;



for j = 1:pathPoints
  repeatCheck = j
  ofPoints = pathPoints
  thisPoint = Path(j, :);
  for l = (j+1):pathPoints
    checkPoint = Path(l, :);
    if (norm(thisPoint - checkPoint) < 10^(-12))
        badIndicesFound = badIndicesFound + 1;
        repeatIndices(badIndicesFound, 1:2) = [j, l];
        repeatPoints(badIndicesFound, 1:3) = thisPoint;
        repeat_uParameters(badIndicesFound, 1:2) = uParSpacePath(j, 1:2);
        repeat_sParameters(badIndicesFound, 1:2) = sParSpacePath(j, 1:2);
    end    
  end
end

check_badIndices = badIndicesFound

%figure
%hold on

%plot3(intersectPlane_u(:,1), intersectPlane_u(:,2), intersectPlane_u(:, 3), 'b.');
%plot3(intersectPlane_s(:,1), intersectPlane_s(:,2), intersectPlane_s(:, 3), 'r.');
%plot3(x0, x0, x0, 'g*')
%plot3(x1, x1, x1, 'g*')
%plot3(Path(:,1), Path(:,2), Path(:,3), 'g.');
%plot3(repeatPoints(:,1), repeatPoints(:,2), repeatPoints(:,3), 'm*');

%figure
%hold on
%plot(uParSpacePath(:,1), uParSpacePath(:,2), 'b.');
%plot(repeat_uParameters(:,1), repeat_uParameters(:,2), 'm*');


%figure
%hold on
%plot(sParSpacePath(:,1), sParSpacePath(:,2), 'r.');
%plot(repeat_sParameters(:,1), repeat_sParameters(:,2), 'm*');
%X = U



%Save Data:
save Path_tryI.dat Path -ascii
save uParSpacePath_tryI.dat uParSpacePath -ascii
save sParSpacePath_tryI.dat sParSpacePath -ascii
save Ds_tryI.dat Ds -ascii
save Du_tryI.dat Du -ascii

%================================================================
%Plotting:
%================================================================


figure
hold on

plot3(intersectPlane_u(:,1), intersectPlane_u(:,2), intersectPlane_u(:, 3), 'b.');
plot3(intersectPlane_s(:,1), intersectPlane_s(:,2), intersectPlane_s(:, 3), 'r.');
plot3(x0, x0, x0, 'g*')
plot3(x1, x1, x1, 'g*')
plot3(intersectionGuesses_u(:,1), intersectionGuesses_u(:,2), intersectionGuesses_u(:,3), 'ko');
plot3(intersectionGuesses_s(:,1), intersectionGuesses_s(:,2), intersectionGuesses_s(:,3), 'mo');



figure
hold on

plot3(intersectPlane_u(:,1), intersectPlane_u(:,2), intersectPlane_u(:, 3), 'b.');
plot3(intersectPlane_s(:,1), intersectPlane_s(:,2), intersectPlane_s(:, 3), 'r.');
plot3(x0, x0, x0, 'g*')
plot3(x1, x1, x1, 'g*')
plot3(intersectionGuesses_u(:,1), intersectionGuesses_u(:,2), intersectionGuesses_u(:,3), 'ko');
plot3(intersectionGuesses_s(:,1), intersectionGuesses_s(:,2), intersectionGuesses_s(:,3), 'mo');

plot3(continuedPath(:, 1), continuedPath(:, 2), continuedPath(:, 3), 'g.');
plot3(continuedPathInv(:, 1), continuedPathInv(:, 2), continuedPathInv(:, 3), 'g.');


figure
hold on
plot(Du(:,1), Du(:,2), 'b.');
plot(uParSpacePath(:,1), uParSpacePath(:,2), 'g.');
plot(u_guessField(:, 1), u_guessField(:, 2), 'ro');
plot(new_guesses_uParameters(:, 1), new_guesses_uParameters(:, 2), 'k*');
axis([-R R -R R]);

figure
hold on
plot(Ds(:,1), Ds(:,2), 'r.');
plot(sParSpacePath(:,1), sParSpacePath(:,2), 'g.');
plot(s_guessField(:, 1), s_guessField(:, 2), 'bo');
plot(new_guesses_sParameters(:, 1), new_guesses_sParameters(:, 2), 'k*');
axis([-R R -R R]);



bound1 = max(Wu_loc);
bound1 = max(bound1);
bound2 = max(Ws_loc);
bound2 = max(bound2);
bound = max([bound1, bound2]);

Ds_maxs = max(Ds);
Ds_mins = min(Ds);
Du_maxs = max(Du);
Du_mins = min(Du);

xu_min = Du_mins(1);
yu_min = Du_mins(2);
xu_max = Du_maxs(1);
yu_max = Du_maxs(2);

xs_min = Ds_mins(1);
ys_min = Ds_mins(2);
xs_max = Ds_maxs(1);
ys_max = Ds_maxs(2);

x_min = min([xu_min, xs_min]);
x_max = max([xu_max, xs_max]);
y_min = min([yu_min, ys_min]);
y_max = max([yu_max, ys_max]);


%================================================================
%Results:
%================================================================


%Phase space plots near zero:
figure
hold on
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'r.'); 
plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'b.'); 
plot3(Path(:,1), Path(:,2), Path(:,3), 'g*');

plot3(continuedPath(:, 1), continuedPath(:, 2), continuedPath(:, 3), 'g*');
plot3(continuedPathInv(:, 1), continuedPathInv(:, 2), continuedPathInv(:, 3), 'g*');

axis([-5 5 -5 5 -5 5])

%Phase space plot (global):
figure
hold on
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'r.'); 
plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'b.'); 
plot3(Path(:,1), Path(:,2), Path(:,3), 'g*');


%Phase space plots near zero:
figure
hold on
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

plot3(Wu(:,1), Wu(:,2), Wu(:,3), 'r.'); 
plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'b.'); 

axis([-5 5 -5 5 -5 5])

%Phase space plot (global):
figure
hold on
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

plot3(Wu(:,1), Wu(:,2), Wu(:,3), 'r.'); 
plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'b.'); 

plot3(Path(:,1), Path(:,2), Path(:,3), 'g*');

plot3(continuedPath(:, 1), continuedPath(:, 2), continuedPath(:, 3), 'g*');
plot3(continuedPathInv(:, 1), continuedPathInv(:, 2), continuedPathInv(:, 3), 'g*');

%Point q is a point on the plane:
qII = p0 + 0.5*(p1-p0);

%eta one and two are the direction vectors of the plane:        
eta1II = [1; 0; -1];
eta2II = p1 - p0;
  
%Iterate Disks:        

Wu_section2 = iterate_set_k(Wu_loc, 75, alpha, tau, a, b, c);
Ws_section2 = inverse_iterate_set_k(Ws_loc, 75, alpha, tau, a, b, c);

%=========================================================================
%Set Operations:
%=========================================================================

sectionIIPlane_u = setIntersectPlane(Wu_section2, eta1II, eta2II, qII, intersectionTol);
sectionIIPlane_s = setIntersectPlane(Ws_section2, eta1II, eta2II, qII, intersectionTol);

figure
hold on

plot3(sectionIIPlane_u(:,1), sectionIIPlane_u(:,2), sectionIIPlane_u(:, 3), 'b.');
plot3(sectionIIPlane_s(:,1), sectionIIPlane_s(:,2), sectionIIPlane_s(:, 3), 'r.');
plot3(x0, x0, x0, 'g*')
plot3(x1, x1, x1, 'g*')



%Surface Plots: --------------------------------------------------------------------



disk_u = unique(Wu_loc, 'rows');
disk_s = unique(Ws_loc, 'rows');



%% Run  program
[tu]=Delaunay2_5D(disk_u);
[ts]=Delaunay2_5D(disk_s);

figure
hold on
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

ax_lim = 2.0;

axis([-ax_lim ax_lim -ax_lim ax_lim -ax_lim ax_lim])
trisurf(tu, disk_u(:,1), disk_u(:,2), disk_u(:,3), 'SpecularColorReflectance', 0.99, ...
                                    'FaceAlpha', 1.0, 'BackFaceLighting', 'reverselit',...
                                    'FaceLighting', 'gouraud', 'facecolor','b','edgecolor','none')

trisurf(ts, disk_s(:,1), disk_s(:,2), disk_s(:,3), 'SpecularColorReflectance', 0.99, ...
                                    'FaceAlpha', 0.75, 'BackFaceLighting', 'reverselit',...
                                    'FaceLighting', 'gouraud', 'facecolor','r','edgecolor','none')

                                
figure
hold on
plot3(continuedPath(:, 1), continuedPath(:, 2), continuedPath(:, 3), 'g.');
plot3(continuedPathInv(:, 1), continuedPathInv(:, 2), continuedPathInv(:, 3), 'g.');


camlight headlight
                            























