
clear
format long 

%=================================================================
%   Compute Two Dimensional Stable and Unstable Manifolds by
%   Parametrization Method.
%
%
%
%
%================================================================



%==============================================================
%Computation Controls:
%==============================================================

%Map Parameters:
a = -0.5;
b = 1.0;
c = 0.5;

alpha = -0.5;
tau = 3.0;

%NOTE:  For alpha = -0.5 and tau = 3.0 p0 is stable
%and p1 is unstable.  In the stable case the first eigenvalue
%is real and the second two are complex.  For the unstable case
%the the first two eigenvalues are complex and the third is real.


%polynomial approximation of W^s(p1):

  %Order of manifold computation
  N = 35;

  %scale the eigenvectors (determins paramatrization uniquely):
  scale_s1 = 0.50;
  scale_s2 = 0.50;  
  
  %number of poin
  num_t = 45;
  num_s = 45;

  s0 = -20;
  t0 = -20;

  t_length = abs(2*t0);
  s_length = abs(2*s0);

  h2 = s_length/num_s;
  h1 = t_length/num_t;

  
  
  
  
  
%==============================================================
%Program:
%==============================================================



%Fixed Points, Eigenvalues, and Eigenvectors:  ================

x0 = (1/2)*(-tau + sqrt( tau^2 - 4*alpha));
x1 = (1/2)*(-tau - sqrt( tau^2 - 4*alpha));

p0 = [x0; x0; x0];
p1 = [x1; x1; x1];

fp0 = henon3D(p0, alpha, tau, a, b, c);
fp1 = henon3D(p1, alpha, tau, a, b, c);

%Differential at p0;

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    
%Differential of p1;

A_1 = (2*a + b)*x1 + tau;
B_1 = (2*c + b)*x1;

Df1 = [ A_1, B_1, 1;
          1,   0, 0;
          0,   1, 0];
    
%Eigenvalues and vectors of Df0:

[E0, eigenvalues_0] = eigs(Df0);

E0
eigenvalues_0

%Eigenvalues and vectors of Df1:

[E1, eigenvalues_1] = eigs(Df1);

E1
eigenvalues_1

%Check linear stablilty of the fixed points

stability_0 = norm(eigenvalues_0(2,2))
stability_1 = norm(eigenvalues_1(1,1))



%==============================================================
%    Computations:
%==============================================================

%Compute unstable manifold of p1:

lambda = eigenvalues_0(2,2)
nu = scale_s1*E0(:, 2)
nu_bar = scale_s2*E0(:, 3)


%initialize the coefficent matrices:
As = zeros(N+1, N+1);
Bs = As;
Cs = As;

%Constant Terms:
As(1,1) = p0(1);
Bs(1,1) = p0(2);
Cs(1,1) = p0(3);

As(2,1) = nu(1);
As(1,2) = nu_bar(1);

Bs(2,1) = nu(2);
Bs(1,2) = nu_bar(2);

Cs(2,1) = nu(3); 
Cs(1,2) = nu_bar(3);


%Compute the i = 0 coefficents:
for j = 2:N
    A_0j = A(0, j, p0, lambda, tau, a, b, c);
    s_0j = s(0,j, As, Bs, a, b, c);
    b_0j = [s_0j; 0 ; 0];
    coef_0j = linsolve(A_0j, b_0j);
    As(1,j+1) = coef_0j(1);
    Bs(1, j+1) = coef_0j(2);
    Cs(1, j+1) = coef_0j(3);
end    

%Compute the j = 0 coefficents:
for i = 2:N
    A_i0 = A(i, 0, p0, lambda, tau, a, b, c);
    s_i0 = s(i, 0, As, Bs, a, b, c);
    b_i0 = [s_i0; 0 ; 0];
    coef_i0 = linsolve(A_i0, b_i0);
    As(i+1, 1) = coef_i0(1);
    Bs(i+1, 1) = coef_i0(2);
    Cs(i+1, 1) = coef_i0(3);
end    


for j = 1:N
    for i = 1:N
        A_ij = A(i, j, p0, lambda, tau, a, b, c);
        s_ij = s(i, j, As, Bs, a, b, c);
        b_ij = [s_ij; 0 ; 0];
        coef_ij = linsolve(A_ij, b_ij);
        As(i+1, j+1) = coef_ij(1);
        Bs(i+1, j+1) = coef_ij(2);
        Cs(i+1, j+1) = coef_ij(3); 
    end
end    


%Evaluate the polynomial to compute the manifold:

place = 1;
i = sqrt(-1);

for n = 0:num_t
    t = t0 + n*h1;
    for m = 0:num_s
       s = s0 + m*h2;
       Ws(place, 1:3) = evaluate(s+(i*t), s-(i*t), As, Bs, Cs, N); 
       place = place + 1;
    end
end

%Evaluate the E (error) functional:

%s = 1.0;
%t = 1.0;

v = s0 + i*t0
w = s0 - (i*t0)

L = [lambda, 0;
         0, conj(lambda)]
z = [v; w]

Lz = L*z
PLz = evaluate(Lz(1), Lz(2), As, Bs, Cs, N)'
PLz = real(PLz)
FPLz = inv_henon3D(PLz, alpha, tau, a, b, c)
Pz = evaluate(v, w, As, Bs, Cs, N)'
         
Es = FPLz - Pz

errorNorm = norm(Es)


roundOffMatrix = imag(Ws);
roundOffNorm = norm(roundOffMatrix)
Ws = real(Ws);

%Plotting:

figure
hold on
   
plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'r.')
plot3(x0, x0, x0, 'go')
plot3(x1, x1, x1, 'ro')


axis([-5 5 -5 5 -5 5])
